<?php

/**
 * @file
 * Tokens for GCC.
 */

/**
 * Implements hook_token_info().
 */
function gcc_token_info() {

  $token_list = array();

  $token_list['gcc_member_count'] = array(
    'name' => t('Total member count'),
    'description' => t('The current entity total group member count if present.'),
  );

  $token_list['gcc_active_member_count'] = array(
    'name' => t('Active member count'),
    'description' => t('The current entity active group member count if present.'),
  );

  $token_list['gcc_blocked_member_count'] = array(
    'name' => t('Blocked member count'),
    'description' => t('The current entity blocked group member count if present.'),
  );

  $token_list['gcc_pending_member_count'] = array(
    'name' => t('Pending member count'),
    'description' => t('The current entity pending group member count if present.'),
  );

  $tokens = array();
  $info = entity_get_info();

  foreach ($info as $entity_type => $type_info) {
    if (isset($type_info['gcc'])) {

      $token_type = isset($type_info['token type']) ? $type_info['token type'] : $entity_type;
      $tokens[$token_type] = $token_list;
    }
  }

  return array('tokens' => $tokens);
}

/**
 * Implements hook_tokens().
 */
function gcc_tokens($type, $tokens, array $data = array(), array $options = array()) {

  $replacements = array();

  $info = entity_get_info();
  $token_entity_types = array();

  foreach ($info as $entity_type => $type_info) {
    if (isset($type_info['gcc'])) {

      $token_type = isset($type_info['token type']) ? $type_info['token type'] : $entity_type;
      $token_entity_types[$token_type] = $entity_type;
    }
  }

  // @fixme: Only supports one level nesting with group_node.
  // @fixme: Need to provide ability to get items recursively using EntityWrapper.
  foreach ($tokens as $name => $token) {
    // Check if we have group-node property with some nesting in it.
    if (isset($data['entity']) && isset($data['entity_type']) && preg_match('/^(field-gcc-audience)\:group-node\:([^\:]+)/i', $token, $reg)) {

      // Yeah it's quite funny.
      $field = str_replace(array('-'), array('_'), $reg[1]);
      $property = $reg[2];
      $wrapper = entity_metadata_wrapper($data['entity_type'], $data['entity']);
      $value = $wrapper->{$field}->group_node->{$property}->value();
      if ($value && !is_object($value) && !is_array($value)) {
        $replacements[$token] = (string) $value;
      }
    }
  }

  if ((!isset($token_entity_types[$type])) || (!isset($data[$type]))) {
    return $replacements;
  }

  $entity_type = $token_entity_types[$type];
  $entity = $data[$type];
  $entity_id = isset($entity->{$info[$entity_type]['entity keys']['id']}) ? $entity->{$info[$entity_type]['entity keys']['id']} : NULL;
  if (empty($entity_id)) {
    return $replacements;
  }

  $is_group = gcc_is_group($entity);

  foreach ($tokens as $name => $original) {
    switch ($name) {
      case 'gcc_member_count':
        $replacements[$original] = $is_group ? gcc_group_get_member_count($entity_type, $entity_id, array()) : '';
        break;

      case 'gcc_active_member_count':
        $replacements[$original] = $is_group ? gcc_group_get_member_count($entity_type, $entity_id, array(GCC_ACTIVE)) : '';
        break;

      case 'gcc_blocked_member_count':
        $replacements[$original] = $is_group ? gcc_group_get_member_count($entity_type, $entity_id, array(GCC_BLOCKED)) : '';
        break;

      case 'gcc_pending_member_count':
        $replacements[$original] = $is_group ? gcc_group_get_member_count($entity_type, $entity_id, array(GCC_PENDING)) : '';
        break;
    }
  }

  return $replacements;
}
