<?php

/**
 * @file
 * Tokens for GCC Context.
 */

/**
 * Implements hook_token_info().
 */
function gcc_context_token_info() {

  $types = array(
    'name' => t('GCC Context'),
    'description' => t('Tokens based on the current GCC Context.'),
  );

  $tokens['entity_id'] = array(
    'name' => t('Entity ID'),
    'description' => t('The current context entity id if present.'),
  );

  $tokens['entity_type'] = array(
    'name' => t('Entity Type'),
    'description' => t('The current context entity type if present.'),
  );

  $tokens['member_count'] = array(
    'name' => t('Total member count'),
    'description' => t('The current context total group member count if present.'),
  );

  $tokens['active_member_count'] = array(
    'name' => t('Active member count'),
    'description' => t('The current context active group member count if present.'),
  );

  $tokens['blocked_member_count'] = array(
    'name' => t('Blocked member count'),
    'description' => t('The current context blocked group member count if present.'),
  );

  $tokens['pending_member_count'] = array(
    'name' => t('Pending member count'),
    'description' => t('The current context pending group member count if present.'),
  );

  return array(
    'types' => array('gcc_context' => $types),
    'tokens' => array('gcc_context' => $tokens),
  );
}

/**
 * Implements hook_tokens().
 */
function gcc_context_tokens($type, $tokens, array $data = array(), array $options = array()) {

  if ($type != 'gcc_context') {
    return array();
  }

  $replacements = array();
  $context = gcc_context_get_context();

  foreach ($tokens as $name => $original) {
    switch ($name) {

      case 'entity_id':
        $replacements[$original] = $context ? $context['entity_id'] : '';
        break;

      case 'entity_type':
        $replacements[$original] = $context ? $context['entity_type'] : '';
        break;

      case 'member_count':
        $replacements[$original] = $context ? gcc_group_get_member_count($context['entity_type'], $context['entity_id'], array()) : '';
        break;

      case 'active_member_count':
        $replacements[$original] = $context ? gcc_group_get_member_count($context['entity_type'], $context['entity_id'], array(GCC_ACTIVE)) : '';
        break;

      case 'blocked_member_count':
        $replacements[$original] = $context ? gcc_group_get_member_count($context['entity_type'], $context['entity_id'], array(GCC_BLOCKED)) : '';
        break;

      case 'pending_member_count':
        $replacements[$original] = $context ? gcc_group_get_member_count($context['entity_type'], $context['entity_id'], array(GCC_PENDING)) : '';
        break;
    }
  }

  return $replacements;
}
