<?php

/**
 * @file
 * Defines and handles the fields extra bonuses.
 */

/* Formatters */

/**
 * Implements hook_field_formatter_info().
 */
function gcc_extra_field_formatter_info() {

  return array(

    'gcc_audience_formatter_label' => array(

      'label' => t('Label'),
      'description' => t('Display the label of the referenced entities.'),
      'field types' => array(GCC_FIELD_TYPE_AUDIENCE),
      'settings' => array(
        'link' => FALSE,
      ),
    ),
  );
}

/**
 * Implements hook_field_formatter_settings_form().
 */
function gcc_extra_field_formatter_settings_form($field, $instance, $view_mode, $form, &$form_state) {
  $display = $instance['display'][$view_mode];
  $element = array();

  if ($display['type'] == 'gcc_audience_formatter_label') {
    $settings = $display['settings'];
    $element['link'] = array(
      '#title' => t('Link label to the referenced entity'),
      '#type' => 'checkbox',
      '#default_value' => $settings['link'],
    );
  }

  return $element;
}

/**
 * Implements hook_field_formatter_settings_summary().
 */
function gcc_extra_field_formatter_settings_summary($field, $instance, $view_mode) {

  $display = $instance['display'][$view_mode];
  $settings = $display['settings'];

  $summary = array();

  if ($display['type'] == 'gcc_audience_formatter_label') {
    $summary[] = $settings['link'] ? t('Link to the referenced entity') : t('No link');
  }

  return implode('<br />', $summary);
}

/**
 * Implements hook_field_formatter_prepare_view().
 */
function gcc_extra_field_formatter_prepare_view($entity_type, $entities, $field, $instances, $langcode, &$items, $displays) {

  $group_ids = array();

  // Collect every possible entity attached to any of the entities.
  foreach ($entities as $id => $entity) {
    if ($displays[$id]['type'] == 'gcc_audience_formatter_label') {
      foreach ($items[$id] as $delta => $item) {
        if (isset($item['entity_type']) && isset($item['entity_id'])) {
          $group_ids[$item['entity_type']][] = $item['entity_id'];
        }
      }
    }
  }

  $group_entities = array();
  if ($group_ids) {
    foreach ($group_ids as $group_entity_type => $ids) {
      $group_entities[$group_entity_type] = entity_load($group_entity_type, $ids);
    }
  }

  // Iterate through the fieldable entities again to attach the loaded data.
  foreach ($entities as $id => $entity) {
    if ($displays[$id]['type'] == 'gcc_audience_formatter_label') {

      $rekey = FALSE;

      foreach ($items[$id] as $delta => $item) {
        if (isset($group_entities[$item['entity_type']][$item['entity_id']])) {

          $items[$id][$delta]['entity'] = $group_entities[$item['entity_type']][$item['entity_id']];
          $items[$id][$delta]['access'] = entity_access('view', $item['entity_type'], $group_entities[$item['entity_type']][$item['entity_id']]);
        }
        else {

          unset($items[$id][$delta]);
          $rekey = TRUE;
        }
      }

      if ($rekey) {
        // Rekey the items array.
        $items[$id] = array_values($items[$id]);
      }
    }
  }
}

/**
 * Implements hook_field_formatter_view().
 */
function gcc_extra_field_formatter_view($entity_type, $entity, $field, $instance, $langcode, $items, $display) {

  $element = array();

  switch ($display['type']) {

    case 'gcc_audience_formatter_label':
      foreach ($items as $delta => $item) {

        if (empty($item['access'])) {
          continue;
        }

        $group_entity = $item['entity'];
        $label = entity_label($item['entity_type'], $group_entity);

        if ($display['settings']['link']) {

          $uri = entity_uri($item['entity_type'], $group_entity);

          if ($uri) {
            $element[] = array(
              '#markup' => l($label, $uri['path'], $uri['options']),
            );
          }
          else {
            $element[] = array('#markup' => check_plain($label));
          }
        }
        else {
          $element[] = array('#markup' => check_plain($label));
        }
      }
      break;
  }

  return $element;
}
