<?php

/**
 * @file
 * GCC Membership Views integration.
 */

/**
 * Implements hook_views_data_alter().
 */
function gcc_membership_views_views_data_alter(&$data) {

  $field_membership = field_info_field(GCC_MEMBERSHIP_FIELD_MEMBERSHIP_GROUP);
  $membership_table = _field_sql_storage_tablename($field_membership);
  $membership_entity_type_column = _field_sql_storage_columnname(GCC_MEMBERSHIP_FIELD_MEMBERSHIP_GROUP, 'entity_type');
  $membership_entity_id_column = _field_sql_storage_columnname(GCC_MEMBERSHIP_FIELD_MEMBERSHIP_GROUP, 'entity_id');

  $entity_info = entity_get_info();

  foreach ($entity_info as $entity_type => $info) {

    // Skip non GCC Entity.
    if (!isset($info['gcc'])) {
      continue;
    }

    // If the entity can be used as a group.
    if ($info['gcc']['group']) {

      // Link the group audience field to the group.
      $data[$membership_table]['group_' . $entity_type] = array(

        'group' => t('GCC'),
        'relationship' => array(

          'title' => t('Groups from membership information (@entity_type)', array('@entity_type' => $entity_info[$entity_type]['label'])),
          'label' => t('Groups from membership information (@entity_type)', array('@entity_type' => $entity_info[$entity_type]['label'])),
          'help' => t('Relate the groups of type @entity_type this membership information refers to.', array('@entity_type' => $entity_info[$entity_type]['label'])),
          'base' => $entity_info[$entity_type]['base table'],
          'base field' => $entity_info[$entity_type]['entity keys']['id'],
          'relationship field' => $membership_entity_id_column,
          'extra' => array(
            array(
              'table' => $membership_table,
              'field' => $membership_entity_type_column,
              'value' => $entity_type,
            ),
          ),
        ),
      );
    }
  }

  $data['gcc_membership']['membership_information'] = array(

    'group' => t('GCC'),
    'relationship' => array(

      'title' => t('Membership information'),
      'label' => t('Membership information'),
      'help' => t('Relate the position this membership information refers to.'),
      'handler' => 'views_handler_relationship_gcc_membership_information',
      'base' => 'node',
      'base field' => 'uid',
      'relationship field' => 'uid',
      'group_ref_field' => 'field_data_' . GCC_MEMBERSHIP_FIELD_MEMBERSHIP_GROUP,
      'group_ref_field_column' => GCC_MEMBERSHIP_FIELD_MEMBERSHIP_GROUP . '_entity_id',
    ),
  );
}
