<?php

/**
 * @file
 * TODO.
 */

$plugin = array(

  'title' => t("GCC group: content's group type and bundle"),
  'description' => t("Control access by the content's group type and bundle."),
  'callback' => 'gcc_panels_gcc_content_group_bundle_access_check',
  'settings form' => 'gcc_panels_gcc_content_group_bundle_access_settings',
  'summary' => 'gcc_panels_gcc_content_group_bundle_access_summary',
  'default' => array('group_types' => array()),
  'required context' => new ctools_context_required(t('Group content'), 'entity'),
);

/**
 * Check for access.
 */
function gcc_panels_gcc_content_group_bundle_access_check($conf, $context, $plugin) {
  if (empty($context->data)) {
    return FALSE;
  }
  $entity = $context->data;

  if (!isset($entity->{GCC_FIELD_AUDIENCE})) {
    return FALSE;
  }

  if (empty($conf['group_types'])) {
    return FALSE;
  }

  $group_types = array();
  foreach ($conf['group_types'] as $group_type) {
    list($group_entity_type, $group_bundle) = explode(':', $group_type);
    $group_types[$group_entity_type][$group_bundle] = TRUE;
  }

  foreach ($entity->{GCC_FIELD_AUDIENCE} as $values) {
    foreach ($values as $value) {
      $group_entity_type = $value['entity_type'];
      if (empty($group_types[$group_entity_type])) {
        continue;
      }
      $group_entity_id = $value['entity_id'];
      $group_entities = entity_load($group_entity_type, array($group_entity_id));
      if (empty($group_entities)) {
        continue;
      }
      list(,, $group_bundle) = entity_extract_ids($group_entity_type, reset($group_entities));
      if (isset($group_types[$group_entity_type][$group_bundle])) {
        return TRUE;
      }
    }
  }

  return FALSE;
}

/**
 * Settings form.
 */
function gcc_panels_gcc_content_group_bundle_access_settings($form, &$form_state, $conf) {
  $group_types = array();
  $bundles = gcc_features_get_enabled_bundles('group');
  foreach ($bundles as $entity_type => $list) {
    $entity_info = entity_get_info($entity_type);
    foreach ($list as $bundle => $label) {
      $group_types[$entity_info['label']][$entity_type . ':' . $bundle] = $label;
    }
  }

  $form['settings']['group_types'] = array(
    '#title' => t('Group node types'),
    '#type' => 'select',
    '#multiple' => TRUE,
    '#size' => 6,
    '#options' => $group_types,
    '#default_value' => $conf['group_types'],
  );

  return $form;
}

/**
 * Provide a summary description.
 */
function gcc_panels_gcc_content_group_bundle_access_summary($conf, $context, $plugin) {
  if (empty($conf['group_types'])) {
    return FALSE;
  }

  $group_types = array();
  foreach ($conf['group_types'] as $group_type) {
    list($group_entity_type, $group_bundle) = explode(':', $group_type);
    $group_types[$group_entity_type][$group_bundle] = TRUE;
  }

  $type_str = array();
  foreach ($group_types as $entity_type => $bundles) {
    $entity_info = entity_get_info($entity_type);
    $bundle_labels = array();
    foreach (array_keys($bundles) as $bundle) {
      $bundle_labels[] = $entity_info['bundles'][$bundle]['label'];
    }
    $type_str[] = $entity_info['label'] . ' (' . implode(', ', $bundle_labels) . ')';
  }

  return t("Content's group type is one of @entity_types", array('@entity_types' => implode(', ', $type_str)));
}
