<?php

/**
 * @file
 * Rules hooks implementations.
 */

/**
 * GCC rules access callback.
 */
function gcc_rules_access_config($type, $name) {

  // Grant everyone access to conditions.
  return $type == 'condition' || user_access('administer gcc configuration');
}

/**
 * Fixme : todo comment.
 */
function gcc_rules_membership_states() {

  return array(

    GCC_PENDING => t('Pending'),
    GCC_ACTIVE => t('Active'),
    GCC_BLOCKED => t('Blocked'),
  );
}

/**
 * Fixme : todo comment.
 */
function gcc_rules_group_roles() {

  $roles = array();
  $role_info = gcc_role_get_list(GCC_GLOBAL_TYPE, GCC_GLOBAL_ID);

  foreach ($role_info as $rid => $role) {
    $roles[$rid] = $role->name;
  }

  return $roles;
}

/**
 * Implements hook_rules_event_info().
 */
function gcc_rules_rules_event_info() {

  $items = array();

  $items['gcc_membership_create'] = array(

    'label' => t('After adding a user to group'),
    'group' => t('GCC groups'),
    'variables' => array(

      'group' => array('type' => 'entity', 'label' => t('Group')),
      'user' => array('type' => 'user', 'label' => t('User')),
      'membership_state' => array(
        'type' => 'integer',
        'label' => t('Membership state'),
        'options list' => 'gcc_rules_membership_states',
      ),
    ),
    'access callback' => 'gcc_rules_access_config',
  );

  $items['gcc_membership_activate'] = array(

    'label' => t('After activating a user in group'),
    'group' => t('GCC groups'),
    'variables' => array(

      'group' => array('type' => 'entity', 'label' => t('Group')),
      'user' => array('type' => 'user', 'label' => t('User')),
    ),
    'access callback' => 'gcc_rules_access_config',
  );

  $items['gcc_membership_block'] = array(

    'label' => t('After disabling a user in group'),
    'group' => t('GCC groups'),
    'variables' => array(

      'group' => array('type' => 'entity', 'label' => t('Group')),
      'user' => array('type' => 'user', 'label' => t('User')),
    ),
    'access callback' => 'gcc_rules_access_config',
  );

  $items['gcc_membership_delete'] = array(

    'label' => t('After deleting a user from group'),
    'group' => t('GCC groups'),
    'variables' => array(

      'group' => array('type' => 'entity', 'label' => t('Group')),
      'user' => array('type' => 'user', 'label' => t('User')),
    ),
    'access callback' => 'gcc_rules_access_config',
  );

  $items['gcc_membership_add_role'] = array(

    'label' => t('After adding a group role to a user'),
    'group' => t('GCC groups'),
    'variables' => array(

      'group' => array('type' => 'entity', 'label' => t('Group')),
      'user' => array('type' => 'user', 'label' => t('User')),
      'role' => array(
        'type' => 'integer',
        'label' => t('Role'),
        'options list' => 'gcc_rules_group_roles',
      ),
    ),
    'access callback' => 'gcc_rules_access_config',
  );

  $items['gcc_membership_remove_role'] = array(

    'label' => t('After removing a group role from a user'),
    'group' => t('GCC groups'),
    'variables' => array(

      'group' => array('type' => 'entity', 'label' => t('Group')),
      'user' => array('type' => 'user', 'label' => t('User')),
      'role' => array(
        'type' => 'integer',
        'label' => t('Role'),
        'options list' => 'gcc_rules_group_roles',
      ),
    ),
    'access callback' => 'gcc_rules_access_config',
  );

  return $items;
}

/**
 * Implements hook_rules_action_info().
 */
function gcc_rules_rules_action_info() {

  $items = array();

  $items['gcc_group_content_add'] = array(

    'label' => t('Add entity to group'),
    'group' => t('GCC groups'),
    'parameter' => array(

      'entity' => array(

        // @todo Specify the list of group content entities.
        'type' => 'entity',
        'label' => t('Entity'),
        'description' => t('The group content which is to be added to a group.'),
        'wrapped' => TRUE,
      ),
      'group' => array(

        // @todo Specify the list of group content entities.
        'type' => 'entity',
        'label' => t('Group'),
        'wrapped' => TRUE,
      ),
    ),
    'base' => 'gcc_rules_add_entity_to_group',
    'access callback' => 'gcc_rules_access_config',
  );

  $items['gcc_group_content_remove'] = array(

    'label' => t('Remove entity from group'),
    'group' => t('GCC groups'),
    'parameter' => array(

      'entity' => array(

        // @todo Specify the list of group content entities.
        'type' => 'entity',
        'label' => t('Entity'),
        'description' => t('The entity which is to be removed from a group.'),
        'wrapped' => TRUE,
      ),
      'group' => array(

        // @todo Specify the list of group content entities.
        'type' => 'entity',
        'label' => t('Group'),
        'wrapped' => TRUE,
      ),
    ),
    'base' => 'gcc_rules_remove_entity_from_group',
    'access callback' => 'gcc_rules_access_config',
  );

  $items['gcc_subcribe_user'] = array(

    'label' => '(Deprecated) ' . t('Subscribe user to group'),
    'group' => t('GCC groups'),
    'parameter' => array(

      'user' => array(

        'type' => 'user',
        'label' => t('User'),
        'description' => t('The user who will be subscribed.'),
      ),
      'group' => array(

        'type' => 'entity',
        'label' => t('Group'),
        'wrapped' => TRUE,
      ),
      'membership_state' => array(

        'type' => 'integer',
        'label' => t('Membership state'),
        'restriction' => 'input',
        'options list' => 'gcc_rules_membership_states',
        'default value' => array(GCC_ACTIVE),
      ),
    ),
    'base' => 'gcc_rules_subscribe_user',
    'access callback' => 'gcc_rules_access_config',
  );

  $items['gcc_subscribe_user'] = array(

    'label' => t('Subscribe user to group'),
    'group' => t('GCC groups'),
    'parameter' => array(

      'user' => array(

        'type' => 'user',
        'label' => t('User'),
        'description' => t('The user who will be subscribed.'),
      ),
      'group' => array(

        'type' => 'entity',
        'label' => t('Group'),
        'wrapped' => TRUE,
      ),
      'membership_state' => array(

        'type' => 'integer',
        'label' => t('Membership state'),
        'restriction' => 'input',
        'options list' => 'gcc_rules_membership_states',
        'default value' => array(GCC_ACTIVE),
      ),
    ),
    'base' => 'gcc_rules_subscribe_user',
    'access callback' => 'gcc_rules_access_config',
  );

  $items['gcc_unsubscribe_user'] = array(

    'label' => t('Unsubscribe user from group'),
    'group' => t('GCC groups'),
    'parameter' => array(

      'user' => array(

        'type' => 'user',
        'label' => t('User'),
        'description' => t('The user who will be unsubscribed.'),
      ),
      'group' => array(

        'type' => 'entity',
        'label' => t('Group'),
        'wrapped' => TRUE,
      ),
    ),
    'base' => 'gcc_rules_unsubscribe_user',
    'access callback' => 'gcc_rules_access_config',
  );

  return $items;
}

/**
 * Implements hook_rules_condition_info().
 */
function gcc_rules_rules_condition_info() {
  $items = array();

  $items['gcc_user_in_group'] = array(

    'label' => t('User is group member'),
    'group' => t('GCC groups'),
    'parameter' => array(

      'account' => array(

        'type' => 'user',
        'label' => t('User'),
      ),
      'group' => array(

        'type' => 'entity',
        'label' => t('Group'),
        'wrapped' => TRUE,
      ),
      'states' => array(

        'type' => 'list<integer>',
        'label' => t('Membership states'),
        'restriction' => 'input',
        'options list' => 'gcc_rules_membership_states',
        'optional' => TRUE,
        'default value' => array(GCC_ACTIVE),
      ),
    ),
    'base' => 'gcc_rules_user_in_group',
    'access callback' => 'gcc_rules_access_config',
  );

  $items['gcc_entity_in_group'] = array(

    'label' => t('Entity is in group'),
    'group' => t('GCC groups'),
    'parameter' => array(

      'entity' => array(

        'type' => 'entity',
        'label' => t('Group content entity'),
        'description' => t('The group content which will be checked.'),
      ),
      'group' => array(

        'type' => 'entity',
        'label' => t('Group'),
        'wrapped' => TRUE,
      ),
    ),
    'base' => 'gcc_rules_entity_in_group',
    'access callback' => 'gcc_rules_access_config',
  );

  $items['gcc_entity_is_group'] = array(

    'label' => t('Entity is group'),
    'group' => t('GCC groups'),
    'parameter' => array(
      'entity' => array(
        'type' => 'entity',
        'label' => t('Entity'),
      ),
    ),
    'base' => 'gcc_rules_entity_is_group',
    'access callback' => 'gcc_rules_access_config',
  );

  return $items;
}

/**
 * Fixme : todo comment.
 */
function gcc_rules_add_entity_to_group(EntityDrupalWrapper $entity, EntityDrupalWrapper $group) {

  $entity_data = $entity->value();

  $entity_type = $group->type();
  $entity_id = intval($group->getIdentifier());

  $field_data = array('entity_type' => $entity_type, 'entity_id' => $entity_id);

  if (!empty($entity_data->{GCC_FIELD_AUDIENCE})) {
    foreach ($entity_data->{GCC_FIELD_AUDIENCE} as $lang => $values) {

      $max_delta = -1;
      foreach ($values as $delta => $value) {

        $max_delta = $delta;
        if ($value['entity_type'] === $entity_type && intval($value['entity_id']) === $entity_id) {
          return;
        }
      }

      $entity_data->{GCC_FIELD_AUDIENCE}[$lang][$max_delta + 1] = $field_data;
      $entity->save();

      return;
    }
  }

  $entity_data->{GCC_FIELD_AUDIENCE}[LANGUAGE_NONE][0] = $field_data;
  $entity->save();
}

/**
 * Fixme : todo comment.
 */
function gcc_rules_remove_entity_from_group(EntityDrupalWrapper $entity, EntityDrupalWrapper $group) {

  $entity_data = $entity->value();

  if (!isset($entity_data->{GCC_FIELD_AUDIENCE})) {
    return;
  }

  $entity_type = $group->type();
  $entity_id = intval($group->getIdentifier());

  foreach ($entity_data->{GCC_FIELD_AUDIENCE} as $lang => $values) {
    foreach ($values as $delta => $value) {
      if ($value['entity_type'] === $entity_type && intval($value['entity_id']) === $entity_id) {

        unset($entity_data->{GCC_FIELD_AUDIENCE}[$lang][$delta]);
        $entity->save();

        return;
      }
    }
  }
}

/**
 * Fixme : todo comment.
 */
function gcc_rules_subscribe_user($account, EntityDrupalWrapper $group, $membership_state) {

  if (gcc_membership_load($group->type(), $group->getIdentifier(), $account->uid)) {
    return;
  }

  gcc_membership_create($group->type(), $group->getIdentifier(), $account->uid, $membership_state);
}

/**
 * Fixme : todo comment.
 */
function gcc_rules_unsubscribe_user($account, EntityDrupalWrapper $group) {

  if (gcc_membership_load($group->type(), $group->getIdentifier(), $account->uid)) {
    gcc_membership_delete($group->type(), $group->getIdentifier(), $account->uid);
  }
}

/**
 * Fixme : todo comment.
 */
function gcc_rules_user_in_group($account, EntityDrupalWrapper $group, $states = array(GCC_ACTIVE)) {

  $membership = gcc_membership_load($group->type(), $group->getIdentifier(), $account->uid);

  if ($membership) {
    return in_array($membership->state, $states);
  }

  return FALSE;
}

/**
 * Fixme : todo comment.
 */
function gcc_rules_entity_in_group($entity, EntityDrupalWrapper $group) {

  $entity_type = $group->type();
  $entity_id = intval($group->getIdentifier());

  if (isset($entity->{GCC_FIELD_AUDIENCE})) {
    foreach ($entity->{GCC_FIELD_AUDIENCE} as $values) {
      foreach ($values as $value) {
        if ($value['entity_type'] === $entity_type && intval($value['entity_id']) === $entity_id) {
          return TRUE;
        }
      }
    }
  }

  return FALSE;
}

/**
 * Fixme : todo comment.
 */
function gcc_rules_entity_is_group($entity) {

  return gcc_is_group($entity);
}
